@@ -1,6 +1,5 @@
 import os
 import subprocess
-import time
 import logging
 
 import requests
@@ -32,7 +31,8 @@ from .utils import (
     validate_protocol,
     UseTlsAdapter,
     get_sub_vs_list_from_data,
-    get_api_bool_string)
+    get_api_bool_string,
+    get_dict_entry_as_list)
 
 
 requests.packages.urllib3.disable_warnings()
@@ -127,6 +127,7 @@ class HttpClient(object):
         return response.text
 
     def _get(self, rest_command, parameters=None, headers=None):
+        print("sdfsdfdsffdsfds")
         return self._do_request('GET', rest_command, parameters,
                                 headers=headers)
 
@@ -246,13 +247,18 @@ class LoadMaster(HttpClient):
         if not isinstance(services, list):
             services = [services]
         for service in services:
-            virt_serv = self.build_virtual_service(service, response)
+            master_vs_id = int(service.get('MasterVSID', 0))
+            if master_vs_id != 0:
+                for vs in services:
+                    if vs.get("Index", 0) == master_vs_id:
+                        virt_serv = self.build_virtual_service(service, vs)
+            else:
+                virt_serv = self.build_virtual_service(service, response)
             virtual_services.append(virt_serv)
         return virtual_services
 
     def get_virtual_service(self, index=None, address=None, port=None,
                             protocol=None):
-
         if index is None:
             validate_ip(address)
             validate_port(port)
@@ -285,9 +291,14 @@ class LoadMaster(HttpClient):
         # create vs and rs objects at this point
         # loop through all vss and attach matching real server objects
         for service_data in virtual_services_data:
-            virtual_service = self.build_virtual_service(service_data,
-                                                         response)
-            virtual_services.append(virtual_service)
+            master_vs_id = int(service_data.get('MasterVSID', 0))
+            if master_vs_id != 0:
+                for vs in virtual_services_data:
+                    if vs.get("Index", 0) == master_vs_id:
+                        virt_serv = self.build_virtual_service(service_data, vs)
+            else:
+                virt_serv = self.build_virtual_service(service_data, response)
+            virtual_services.append(virt_serv)
 
         rs_vs_indexes = [(rs['VSIndex'], rs)
                          for rs in global_real_servers_data]
@@ -304,27 +315,32 @@ class LoadMaster(HttpClient):
 
         return virtual_services
 
-    def build_virtual_service(self, service, response=None):
+    def build_virtual_service(self, service, response):
         """Create a VirtualService instance with populated with API parameters
 
+        This does not include potentially attached real servers
         :param service: OrderedDict populated with virtual service data
         :param response: Optional response of a listvs call. This acts as a
-                         cache, if you want to create a lot of VirtualService
-                         objects in a row, such as with looping, you can call
-                         listvs and pass the response in each time and this
-                         will nullify the extra listvs calls.
+        cache, if you want to create a lot of VirtualService
+        objects in a row, such as with looping, you can call
+        listvs and pass the response in each time and this
+        will nullify the extra listvs calls.
         :return: VirtualService object with populated attributes
         """
+        asfdsfda = service.get('MasterVSID')
         is_sub_vs = True if int(service.get('MasterVSID', 0)) != 0 else False
         if is_sub_vs:
-            response = self._get("/showvs", {"vs": service.get('MasterVSID')})
-            data = get_data(response)
-            subvs_data = get_sub_vs_list_from_data(data)[1]
+            print("de subvbs")
+            # Add lb properties to the sub vs
+            #response = self._get("/showvs", {"vs": service.get('MasterVSID')})
+            parent_vs_data = get_data(response)
+            subvs_lb_props = get_sub_vs_list_from_data(parent_vs_data)[1]
             virt_serv = VirtualService(self.access_info, service.get('Index'),
                                        is_sub_vs=True)
-            virt_serv.subvs_data = subvs_data[service.get('Index')]
+            virt_serv.subvs_data = subvs_lb_props[service.get('Index')]
             virt_serv.subvs_data['parentvs'] = service.get('MasterVSID')
         else:
+            # Add any sub VSs to the top level VS
             if response is None:
                 response = self._get("/listvs")
             data = get_data(response)
@@ -334,26 +350,20 @@ class LoadMaster(HttpClient):
                                        service.get('Protocol'),
                                        is_sub_vs=False)
             virt_serv.subvs_entries = []
-            for key, value in data.items():
-                if key == "VS":
-                    if not isinstance(value, list):
-                        value = [value]
-                    vs_index = service.get('Index')
-                    for vs in value:
-                        # add subvs to parent vs
-                        if vs['MasterVSID'] == vs_index:
-                            subvs = VirtualService(self.access_info,
-                                                   vs['Index'])
-                            subvs.populate_default_attributes(vs)
-                            subvs_api_entries = service.get("SubVS", [])
-                            if not isinstance(subvs_api_entries, list):
-                                subvs_api_entries = [subvs_api_entries]
-                            for subvs_api in subvs_api_entries:
-                                if subvs_api["VSIndex"] == subvs.index:
-                                    subvs.subvs_data = subvs_api
-                                    subvs.subvs_data['parentvs'] = vs_index
-
-                            virt_serv.subvs_entries.append(subvs)
+            services = get_dict_entry_as_list("VS", data)
+            this_vs_index = service.get('Index')
+            for vs in services:
+                # add subvs to parent vs
+                if vs['MasterVSID'] == this_vs_index:
+                    subvs = VirtualService(self.access_info, vs['Index'])
+                    subvs.populate_default_attributes(vs)
+                    subvs_api_entries = get_dict_entry_as_list("SubVS", data)
+                    for subvs_api in subvs_api_entries:
+                        # add the "Rs" part of the subvs to the subvs instance
+                        if subvs_api["VSIndex"] == subvs.index:
+                            subvs.subvs_data = subvs_api
+                            subvs.subvs_data['parentvs'] = this_vs_index
+                    virt_serv.subvs_entries.append(subvs)
         virt_serv.populate_default_attributes(service)
         return virt_serv
 
@@ -498,8 +508,7 @@ class LoadMaster(HttpClient):
     def backup(self, location='backups'):
         if not os.path.exists(location):
             os.makedirs(location)
-        file_name = os.path.join(location, "{}_{}.backup".format(
-            self.ip_address, time.strftime("%Y-%m-%d_%H:%M:%S")))
+        file_name = os.path.join(location, "{}.backup".format(self.ip_address))
 
         with open(file_name, 'wb') as file:
             cmd = self._get_curl_command_list('backup')
@@ -888,8 +897,7 @@ class RealServer(HttpClient, KempBaseObjectModel):
 
     def _get_base_parameters(self):
         """Returns the bare minimum VS parameters. IP, port and protocol"""
-        # Comment out this code until vsindex is correct on a showrs call.
-        #if self.rsindex is None:
+        # Can't use the rsindex until vsindex is correct on a showrs call
         return {
             "vs": self.vs,
             "port": self.port,
@@ -897,11 +905,6 @@ class RealServer(HttpClient, KempBaseObjectModel):
             "rs": self.rs,
             "rsport": self.rsport,
         }
-        #else:
-        #    return {
-        #        "vs": self.vsindex,
-        #        "rs": "!{}".format(self.rsindex),
-        #    }
 
     def save(self, update=False):
         if not update:
